@@ -428,7 +428,7 @@ ip0 link del wg0
 
 ! n0 wg show doesnotexist || false
 
-n0 ip link add wg0 type wireguard
+ip0 link add wg0 type wireguard
 n0 wg set wg0 private-key <(echo "$key1") peer "$pub2" preshared-key <(echo "$psk")
 [[ $(n0 wg show wg0 private-key) == "$key1" ]]
 [[ $(n0 wg show wg0 preshared-keys) == "$pub2	$psk" ]]
@@ -444,7 +444,7 @@ n0 wg set wg0 peer "$pub2"
 n0 wg set wg0 private-key <(echo "$key1")
 n0 wg set wg0 peer "$pub2"
 [[ $(n0 wg show wg0 peers) == "$pub2" ]]
-n0 ip link del wg0
+ip0 link del wg0
 
 declare -A objects
 while read -t 0.1 -r line 2>/dev/null || [[ $? -ne 142 ]]; do
