@@ -1,7 +1,10 @@
-/* Copyright (C) 2015-2017 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved. */
+/* SPDX-License-Identifier: GPL-2.0
+ *
+ * Copyright (C) 2015-2018 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved.
+ */
 
-#ifndef WGTIMERS_H
-#define WGTIMERS_H
+#ifndef _WG_TIMERS_H
+#define _WG_TIMERS_H
 
 struct wireguard_peer;
 
@@ -15,4 +18,4 @@ void timers_handshake_complete(struct wireguard_peer *peer);
 void timers_session_derived(struct wireguard_peer *peer);
 void timers_any_authenticated_packet_traversal(struct wireguard_peer *peer);
 
-#endif
+#endif /* _WG_TIMERS_H */
